@@ -1,7 +1,7 @@
 extractAIC.lm.madlib <- function(fit, scale=0, k=2, ...)
 {
     edf <- length(fit$coef)
-    sse <- lookat(sum(resid(fit)^2))[1,1,drop=TRUE]
+    sse <- lookat(sum(resid(fit)^2))
     c(edf, fit$nobs * log(sse/fit$nobs) + k * edf)
 }
 
@@ -9,7 +9,7 @@ extractAIC.lm.madlib <- function(fit, scale=0, k=2, ...)
 
 logLik.lm.madlib <- function(object, ...)
 {
-    sse <- lookat(sum(residuals(object)^2))[1,1,drop=TRUE]
+    sse <- lookat(sum(residuals(object)^2))
     n <- object$nobs
     ll <- 0.5 * (-n * (log(2 * pi) + 1 - log(n) + log(sse)))
     attr(ll, "df") <- length(object$coef) + 1
